==> percona-server-mongodb_4.0.24.orig.tar.gz: percona-server-mongodb-4.0.24-19/jstests/core/list_all_local_cursors.js <==
// $listLocalCursors relies on in-memory state, which may not survive failovers.
// @tags: [
//   does_not_support_stepdowns,
//   uses_testing_only_commands,
// ]

// Basic tests for the $listLocalCursors aggregation stage.

(function() {
    "use strict";

    const admin = db.getSisterDB("admin");
    function listAllCursorsWithId(cursorId) {
        return admin
            .aggregate([
                {"$listLocalCursors": {}},
                {"$match": {"id": cursorId}},
            ])
            .toArray();
    }

    let session = db.getMongo().startSession();
    let testDb = db.getSisterDB("listAllLocalCursors");
    let testDbWithSession = session.getDatabase("listAllLocalCursors");
    testDb.data.drop();
    assert.writeOK(testDb.data.insert({a: 1}));
    assert.writeOK(testDb.data.insert({a: 1}));
    let cursorIdWithSession =
        assert.commandWorked(testDbWithSession.runCommand({find: "data", batchSize: 0})).cursor.id;
    let cursorIdWithoutSession =
        assert.commandWorked(testDb.runCommand({find: "data", batchSize: 0})).cursor.id;

    // Verify that we correctly list the cursor which is outside of a session.
    let foundCursors = listAllCursorsWithId(cursorIdWithoutSession);
    assert.eq(foundCursors.length, 1, tojson(foundCursors));
    assert.eq(foundCursors[0].ns, "listAllLocalCursors.data", tojson(foundCursors));
    assert.eq(foundCursors[0].id, cursorIdWithoutSession, tojson(foundCursors));

    // Verify that we correctly list the cursor which is inside of a session.
    foundCursors = listAllCursorsWithId(cursorIdWithSession);
    assert.eq(foundCursors.length, 1, tojson(foundCursors));
    assert.eq(foundCursors[0].ns, "listAllLocalCursors.data", tojson(foundCursors));
    assert.eq(foundCursors[0].id, cursorIdWithSession, tojson(foundCursors));
    assert(foundCursors[0].hasOwnProperty("lsid"), tojson(foundCursors));
    assert.eq(
        foundCursors[0].lsid.id, session._serverSession.handle.getId().id, tojson(foundCursors));

    assert.commandWorked(testDbWithSession.runCommand(
        {killCursors: "data", cursors: [cursorIdWithSession, cursorIdWithoutSession]}));
    session.endSession();
})();

==> percona-server-mongodb-36_3.6.12.orig.tar.gz: percona-server-mongodb-3.6.12-3.2/jstests/core/list_all_local_cursors.js <==
// Basic tests for the $listLocalCursors aggregation stage.
//
// $listLocalCursors relies on in-memory state, which may not survive failovers.
// Uses features that require featureCompatibilityVersion 3.6.
// @tags: [
//   does_not_support_stepdowns,
//   requires_fcv36,
//   uses_testing_only_commands,
// ]

(function() {
    "use strict";

    const admin = db.getSisterDB("admin");
    const countAllMatchingLocalCursors = function(match) {
        return admin
            .aggregate([
                {"$listLocalCursors": {}},
                {"$match": match},
                {"$count": "matches"},
            ])
            .next()
            .matches;
    };

    let session = db.getMongo().startSession();
    let testDb = db.getSisterDB("listAllLocalCursors");
    let testDbWithSession = session.getDatabase("listAllLocalCursors");
    testDb.data.drop();
    assert.writeOK(testDb.data.insert({a: 1}));
    assert.writeOK(testDb.data.insert({a: 1}));
    let cursorIdWithSession =
        assert.commandWorked(testDbWithSession.runCommand({find: "data", batchSize: 0})).cursor.id;
    let cursorIdWithoutSession =
        assert.commandWorked(testDb.runCommand({find: "data", batchSize: 0})).cursor.id;

    // Ensure that the cache now contains the session and is visible by admin.
    const count = countAllMatchingLocalCursors({
        "ns": "listAllLocalCursors.data",
        "$or": [
            {
              "id": cursorIdWithSession,
              "lsid.id": session._serverSession.handle.getId().id,
            },
            {
              "id": cursorIdWithoutSession,
            },
        ],
    });
    assert.eq(count, 2);

    assert.commandWorked(testDbWithSession.runCommand(
        {killCursors: "data", cursors: [cursorIdWithSession, cursorIdWithoutSession]}));
    session.endSession();
})();
